import os, sys
import maya.cmds as cmds
#import denreiMaya
#from pymel import *

UTGAMEPATH = 'C:/Users/denrei/Documents/My Games/Unreal Tournament 3/UTGame'

#TODO if W is released while S is held, S gets cancelled
#Holding A and D together loses one of them

class MayaKey:
	def __init__(self, string, value, time):
		self.string = string
		self.value = value 
		self.time = time 

def GetInput( logfilename='', desiredSession=-1 ):
#	logfolder = denreiMaya.getInfo( 'UTGamePath' ).rstrip('/') + '/Logs/'
	if logfilename is '':
		logfilename="Launch.log"
		logfolder = UTGAMEPATH.rstrip('/') + '/Logs/'
		logfilenamefull = logfolder + logfilename
	else:
		logfilenamefull = logfilename
	print 'Opening "'+logfilenamefull+'"'
	try:
		logfile = open(logfilenamefull, "r").readlines()
	except EnvironmentError:
		sys.stderr.write('Incorrect UTGame path: "'+logfilenamefull+'"')
		sys.exit(2)

	currentSession = 0
	markerString = 'ScriptLog: MayaKey'
	keys = []

	for line in logfile:
		words = line.split('_')
		if markerString in words[0]:
			foundSession = int(words[0].split( markerString )[1])
		#	foundSession = int(words[0][-1:])
			if foundSession is desiredSession:
				keys.append( MayaKey( words[1], words[2], words[3]) )
			elif desiredSession is -1:
				if foundSession > currentSession:
					keys = []
					currentSession = foundSession 
				keys.append( MayaKey( words[1], words[2], words[3]) )

	objList = ['LeftAnalog_Driver', 'RightAnalog_Driver', 'Spacebar_Driver', 
				'Q_Driver','F_Driver','LeftMouseButton_Driver','RightMouseButton_Driver' ]
	cmds.cutKey( objList )

	cmds.setKeyframe( objList, v=0, at=['tx', 'tz'], t=['0sec'] )
	
	if len(keys) is 0:
		print 'Found No Keys.  Aborting'
		return


	duration = float(keys[len(keys)-1].time)
	endFrame = duration * 30
	cmds.setAttr( 'defaultRenderGlobals.endFrame', endFrame)

	print 'Used Session %i.' %currentSession		
	print 'Found %s Keys.' %len(keys)
	print 'Duration: %s seconds.' %duration
	print 'Frames: %s.' %endFrame

	for key in keys:
#		print "string:"+key.string+" value:"+key.value+" time:" + key.time
		if 'W' is key.string:
			cmds.setKeyframe( 'LeftAnalog_Driver', v=-5*float(key.value), at='translateZ', t=[key.time+'sec'] )
		if 'S' is key.string:
			cmds.setKeyframe( 'LeftAnalog_Driver', v=5*float(key.value), at='translateZ', t=[key.time+'sec'] )
		if 'A' is key.string:
			cmds.setKeyframe( 'LeftAnalog_Driver', v=-5*float(key.value), at='translateX', t=[key.time+'sec'] )
		if 'D' is key.string:
			cmds.setKeyframe( 'LeftAnalog_Driver', v=5*float(key.value), at='translateX', t=[key.time+'sec'] )

#		if 'AnalogX' in key.string:
#			cmds.setKeyframe( 'RightAnalog_Driver', v=0.25*float(key.value), at='translateX', t=[key.time+'sec'] )
#		if 'AnalogY' in key.string:
#			cmds.setKeyframe( 'RightAnalog_Driver', v=-0.25*float(key.value), at='translateZ', t=[key.time+'sec'] )

		if 'MouseX' in key.string:
			cmds.setKeyframe( 'RightAnalog_Driver', v=0.1*float(key.value), at='translateX', t=[key.time+'sec'] )
		if 'MouseY' in key.string:
			cmds.setKeyframe( 'RightAnalog_Driver', v=-0.1*float(key.value), at='translateZ', t=[key.time+'sec'] )			

		if 'F' in key.string:
			cmds.setKeyframe( 'F_Driver', v=10*int(key.value), at='translateX', t=[key.time+'sec'] )
		if 'Q' in key.string:
			cmds.setKeyframe( 'Q_Driver', v=10*int(key.value), at='translateX', t=[key.time+'sec'] )			
		if 'SpaceBar' in key.string:
			cmds.setKeyframe( 'Spacebar_Driver', v=10*int(key.value), at='translateX', t=[key.time+'sec'] )
		if 'LeftMouseButton' in key.string:
			cmds.setKeyframe( 'LeftMouseButton_Driver', v=10*int(key.value), at='translateX', t=[key.time+'sec'] )			
		if 'RightMouseButton' in key.string:
			cmds.setKeyframe( 'RightMouseButton_Driver', v=10*int(key.value), at='translateX', t=[key.time+'sec'] )			
			
	cmds.keyTangent( objList, ott='step', time=('0sec', keys[len(keys)-1].time+'sec') )
	cmds.keyTangent( 'RightAnalog_Driver', itt='spline', ott='spline', time=('0sec', keys[len(keys)-1].time+'sec') )
	NormalizeCurves( 'RightAnalog_Driver', ['tx','tz'], 5 )

def NormalizeCurves( object, attributes, scale=1 ):
	for attribute in attributes:
		largestKey = FindLargestKey( object, attribute )
		cmds.scaleKey( object, valueScale=1/(largestKey/scale),at=attribute )

def FindLargestKey( object, attribute ):
	maxVal = 0 
	for keyTime in cmds.keyframe( object, q=1, at=attribute ):
		val = cmds.keyframe( object, time=(keyTime,keyTime), q=1, at=attribute, ev=1 )[0] or 0
		maxVal = max( maxVal, abs(val) )
	return maxVal

#GetInput()
