"""
These scripts were written by Denrei to assist him in the tons of modeling, animating and texture work he knew he would be doing for CodaMod. The idea is that you can keep working on your original low res and high res objects, and have the changes automatically migrated to the point of exporting into Unreal. You can rapidly make tweaks to the meshes at any stage of the development, and see the end results quickly, without vulnerability to human error. For example, if someone were to create a model, and get it into the engine only to notice some flaws which required them to go back and
some of the UVs on the low res mesh around - this would mean going through the process of rebaking maps, reweighting the mesh and tons of little tasks along the way... which are confusing and tiring to deal with. These scripts allow small and massive changes to be propagated with the click of a button - rather what can end up being hours redoing the same troublesome tedious work. 

import maya.cmds as cmds
import maya.mel as mel
import denreiMaya
denreiMaya.run()

Tutorial:
http://wiki.codamod.com/Denrei_Maya

This is all Denrei's work so don't steal it please.  
You may modify it and mess with it so long as your work is non-commerical and Denrei gets proper credit - but please do not rerelease!
"""


import maya.cmds as cmds
import maya.mel as mel
import sys
import os
libdir = os.path.abspath(os.path.join(os.path.dirname(__file__), 'denreiMaya'))
sys.path.insert(0, libdir)


import actorx, bake, gui, migrate, util
reload(actorx)
reload(bake)
reload(gui)
reload(migrate)
reload(util)
from util import *

sceneInfoNode = 'persp'
rebuildNotifyList = [
				'bCustomFolders',
				'rootJoint',
				'psaFileName',
				'animString',
				'bReadyToModel',
				'bCreateIni',
				]
defaultCharacterInfoList = [
				'charName',
				'description',
				'characterID',
				'familyID',
				'faction',
				'portrait',
				]
defaultLayerList = [
				( 'scrapLayer', 9, 1 ),
				( 'lowResWorkLayer', 13, 0 ), 
				( 'highResWorkLayer',1, 0 ),
				( 'jointLayer',0, 0 ),
				( 'weightSourceLayer',5, 0 ),
				( '_highResProxyLayer',3, 0 ),
				( '_bakeLayer',14, 0 ),
				( '_exportLayer',4, 0 )		
				]

defaultLocatorList = [ 
				( 'lowResWorkLocator',	(0,0,30) ),
				( 'highResWorkLocator',	(0,0,-30) ), 
				( 'jointLocator',		(0,0,-60) ),
				( 'bakeLocator',		(0,0,0) )
				]

def run():
	print 'Run DenreiWindow'
	buildDenreiWindow( )

def buildDenreiWindow():
	return gui.buildDenreiWindow()

def runActorX():
	actorx.run()

def actorXAnimations():
	actorx.actorXAnimations()

def prepareModelingScene():
	selection = cmds.ls( sl=1 )	
	for layer in defaultLayerList:
		createLayer( layer[0], layer[1], layer[2] )
		cleanLayerConnections( layer[0] )
	cams = cmds.ls( typ='camera' ) or []
	for cam in cams:
		cmds.setAttr( cam+'.farClipPlane', 9999 )
	for locator in defaultLocatorList:
		if not cmds.objExists( locator[0] ):
			cmds.spaceLocator( n=locator[0] )
			if locator[0] == sceneInfoNode.partition('Shape')[0]:
				for info in defaultCharacterInfoList:
					setInfo( info )
			cmds.xform( locator[0], s=(30,1,30) )
			cmds.xform( locator[0], ro=(0,-45,0) )
			text = cmds.textCurves( ch=0, f='Times New Roman|h-13|w400|c0', t=locator[0].partition('Locator')[0] )
			cmds.parent( text, locator[0] )
			cmds.xform( locator[0], t=locator[1] )
		cmds.editDisplayLayerMembers( 'scrapLayer', locator[0], nr=1 )
	setInfo( 'bReadyToModel', 1 )

def migrateMeshes( meshes=[] ):
	if( len(meshes) == 0 ):
		meshes = cmds.ls(sl=1) or []

	lowResNodes = []
	highResNodes = []
	createdNodes = []
	exportNodes = []
	
	for mesh in meshes:
		if denreiMaya.isChildFromLayer('lowResWorkLayer', mesh ) == 1:
			lowResNodes.append( mesh )
		if denreiMaya.isChildFromLayer('highResWorkLayer', mesh ) == 1:
			highResNodes.append( mesh )

	if len(lowResNodes) > 0:
		createdNodes.append(migrate.createBakeMesh( lowResNodes ))
		createdNodes.append(migrate.createExportMesh( lowResNodes ))
	if len(highResNodes) > 0:
		createdNodes.append(migrate.createHighResProxy( highResNodes ))
	return createdNodes

#	For ambient/shadow maps...
#	blowInHighProxy()
#	diffuseTextureNode = bake.bakeDiffuseMap( 'diffuseBakeNode', diffuseMapSize )
#	blowOutHighProxy()
#
#	blowOutHighProxy()

def bakeNodes():
	msg = 'No bakeNode found... empty and reassign contents of "_highResProxyLayer" with only meshes and make sure a bakeNode exists from MigrateMeshes'
	if cmds.objExists( 'bakeNode' ) == 0:
		cmds.confirmDialog( title='Denrei Requests...', message=msg )
		return
		
	diffuseTextureNode = bake.bakeDiffuseMap( 'bakeNode', getInfo( 'iDiffuseMapSize' ), getInfo( 'iTextureHeightRatio' ) )
	normalTextureNode = bake.bakeNormalMap( 'bakeNode', getInfo( 'iNormalMapSize' ), getInfo( 'iTextureHeightRatio' ) )
	alphaTextureNode = bake.bakeAlphaMap( 'bakeNode', getInfo( 'iAlphaMapSize' ), getInfo( 'iTextureHeightRatio' ) )

	if getInfo( 'bKeepShaders' ) == 1:
		return

	#TODO check to see that texutre nodes arnt none
	bakeShader = bake.configureShader( normalTextureNode, diffuseTextureNode,alphaTextureNode,'BakeShader' )
	SG = cmds.connectionInfo( bakeShader + '.outColor', dfs=True )[0].partition('.')[0]
	mel.eval( 'sets -e -forceElement '+SG+' '+'bakeNode' )

	melCmd = 'sets -e -forceElement '+SG
	exportNodes = cmds.editDisplayLayerMembers( '_exportLayer', q=1 ) or []
	for node in exportNodes:
		melCmd += ' '+node
	mel.eval( melCmd )
	print melCmd

#	Apparently this procedure cannot be found
#	mel.eval('scOpt_performOneCleanup( { "transformOption" } );')


def blowOutAll():
        blowOut( ['_highResProxyLayer','lowResWorkLayer','highResWorkLayer'],1)
def blowInAll():
        blowOut( ['_highResProxyLayer','lowResWorkLayer','highResWorkLayer'],0)
def blowOut( layers, bBlowOut ):
        for layer in layers:
                nodes = cmds.editDisplayLayerMembers( layer, q=1 ) or []
                for node in nodes:
                        if cmds.listAttr( node, st=['blowOutLocation'] ) == None:
                                cmds.addAttr( node, longName='blowOutLocation', attributeType='double3' )
                                cmds.addAttr( node, longName='X', attributeType='double', parent='blowOutLocation' )
                                cmds.addAttr( node, longName='Y', attributeType='double', parent='blowOutLocation' )
                                cmds.addAttr( node, longName='Z', attributeType='double', parent='blowOutLocation' )

                if bBlowOut == 0:
                        for node in nodes:
                                loc = cmds.getAttr( node + '.t' )[0]
                                if loc[0] != 0 or loc[1] != 0:
                                        cmds.setAttr( node + '.blowOutLocation', loc[0], loc[1], loc[2], typ='double3' )
                                        cmds.setAttr( node + '.tx', 0)
                                        cmds.setAttr( node + '.ty', 0)
                else:
                        for node in nodes:
                                loc = cmds.getAttr(node+'.blowOutLocation')[0]
                                cmds.setAttr( node + '.tx', loc[0]) 
                                cmds.setAttr( node + '.ty', loc[1])
	
def setOutputFolder():
	result = cmds.promptDialog(
			title='Set Output Folder',
			message='Set the folder where your exported assest will be placed',
			text = getOutputFolder(),
			button=['OK', 'Cancel'],
			defaultButton='OK',
			cancelButton='Cancel',
			dismissString='Cancel')
	if result == 'OK':
		string = cmds.promptDialog(query=True, text=True)
		sceneName = mel.eval( 'basenameEx( "' + cmds.file( q=1, sn=1) + '") ' )
		packagesPath = cmds.workspace( q=1, sn=1) + '/packages'
		outputFolder = packagesPath + '/' + sceneName.rstrip( '0123456789' )
		setInfo( 'outputFolder', string )		
	else:
		setInfo( 'outputFolder', '' )

def exportPSD(  ):
	processFiles( cmds.ls( typ='psdFileTex' ) or [] )

def processFiles( fileNodes ):
	print 'processFiles: '
	for fileNode in fileNodes:
		print processTextureNode( fileNode )
	return 'processFiles complete'

def processTextureNode( fileNode ):
	type = cmds.nodeType( fileNode )
	msg = 'Please make sure you are using true "psdFileTex" nodes and not ordinary "file" nodes'
	if selType == 'file':
		cmds.confirmDialog( title='Denrei Requests...', message=msg )
		return msg
	elif selType == 'psdFileTex': 
		return str(bake.PSDsToTGAs( ))

def isReadyToModel():
	for layer in denreiMaya.defaultLayerList:
		if not ( cmds.objExists( layer[0] ) ):
			return 0
	for locator in denreiMaya.defaultLocatorList:
		if not ( cmds.objExists( locator[0] ) ):
			return 0
	return 1
"""
MEL HotKey command:
python("import maya.cmds as cmds;import denreiMaya;reload(denreiMaya);denreiMaya.polySeparatePreserveLayers( cmds.ls(sl=1) );");
"""
def polySeparatePreserveLayers( selection ):
	for node in selection:
		layer = cmds.connectionInfo( node + '.drawOverride', sfd=1 ).partition('.')[0]
		separated = cmds.polySeparate( node, ch=0 )
		if layer != '':
			for item in separated:
				cmds.editDisplayLayerMembers( layer, item, nr=1 )
"""
MEL HotKey command:
python("import maya.cmds as cmds;import denreiMaya;reload(denreiMaya);denreiMaya.polyCombinePreserveLayers( cmds.ls(sl=1) );");
"""
def polyCombinePreserveLayers( selection ):
	masterLayer = ''
	for node in selection:
		layer = cmds.connectionInfo( node + '.drawOverride', sfd=1 ).partition('.')[0]
		if layer != '':
			masterLayer = layer 
	
	united = cmds.polyUnite( selection, ch=0 )
	if masterLayer != '':
		cmds.editDisplayLayerMembers( masterLayer, united, nr=1 )

"""
MEL HotKey command:
python("import denreiMaya;reload(denreiMaya);denreiMaya.processSelection();");

Python Shelf Command: (This is prefered, because it spits the UTCustomChar.ini into the script editor...
import denreiMaya
reload( denreiMaya)
print denreiMaya.processSelection()
"""
# Mirror keyframes at the current time across selected rig objects
# It is assumed that things are named with R_ and L_ 
# If nothing is selected, the entire current character set will be used
def mirrorKeyFrames( selection ):
	if len(selection) == 0:
		selection = getCharacterSetObjects()

	flippedList = []
	for sel in selection:
		if not sel in flippedList:
			selList = sel.partition('_')
			if selList[0] == 'R' or selList[0] == 'L':
				flipKeysBetweenObjects( 'R_'+ selList[2], 'L_'+ selList[2] )
				flippedList.append( 'R_'+ selList[2] )
				flippedList.append( 'L_'+ selList[2] )
			else:
				flipKeysOnObject( sel )
				flippedList.append( sel )

def getCharacterSetObjects():
	objDict = {}
	characterSet = cmds.selectionConnection('highlightList',q=1,object=1 )
	if characterSet is not None:
		members = cmds.character(characterSet, query=True)
		for member in members:
			if cmds.objExists( member ):
				objDict[member.partition('.')[0]] = 1
		return objDict.keys()
	else:
		problem('No character set is active...')

def flipKeysBetweenObjects( src, dest ):
	srcAtt = []
	dstAtt = []
	if cmds.objExists( src ):
		srcAtt = cmds.listAttr( src, k=1, u=1 )
	if cmds.objExists( dest ):
		dstAtt = cmds.listAttr( dest, k=1, u=1 )
	for att in srcAtt:
		if att in dstAtt:
			tmp = cmds.getAttr( dest+'.'+att )
			cmds.setAttr( dest+'.'+att, cmds.getAttr( src+'.'+att ) )
			cmds.setAttr( src+'.'+att, tmp )
	flipKeysOnObject( src )
	flipKeysOnObject( dest )

def flipKeysOnObject( object ):
	objAtt = []
	if cmds.objExists( object ):
		objAtt = cmds.listAttr( object, k=1, u=1, st=['translateX', 'rotateY', 'rotateZ'] )
	for att in objAtt:
		cmds.setAttr( object+'.'+att, cmds.getAttr( object+'.'+att ) * -1 )

def cycleAnimation( ):
	min = getInfo( 'iCycle_min' )
	max = getInfo( 'iCycle_max' )
	cycleLength = max - min
#	end = cmds.playbackOptions(q=1,aet=1)
	end = 999
	objects = getCharacterSetObjects()
	cmds.cutKey( objects, t=(max,end) )
	cmds.copyKey( objects, t=(min,max) )
	cmds.pasteKey( objects, t=(max,max) ) 
	cmds.copyKey( objects,t=(min,max))
	cmds.pasteKey( objects, t=( max + ( max - min ),max + ( max - min ) ) )

	_min = cmds.playbackOptions(q=1,min=1)
	_max = cmds.playbackOptions(q=1,max=1)
	_cur = cmds.currentTime(q=1)
	pasteMax = max + cycleLength + cycleLength

	cmds.playbackOptions(e=1,min=min,max=pasteMax)
	cmds.currentTime( max )

	lastTime = -50
	while cmds.currentTime( q=1 ) < max + cycleLength and lastTime < cmds.currentTime( q=1 ):
		print 't: '+str(cmds.currentTime( q=1 ))
		mirrorKeyFrames( objects )
		nextKey = cmds.findKeyframe( objects, an='objects', which='next' )
		lastTime = cmds.currentTime( q=1 )
		cmds.currentTime( nextKey )

	lastKey = cmds.findKeyframe( objects, an='objects', which='last' )
	cmds.currentTime( lastKey )
	mirrorKeyFrames( objects )
	
	cmds.playbackOptions(e=1,min=_min,max=_max)
	cmds.currentTime( _cur )
	cmds.keyTangent(cmds.ls(type='animCurve'),itt='spline',ott='spline')

def moveToNewSkel():
        migrate.moveToNewSkel()

def disableMaskShaders():
	print "dis"
	bake.disableMaskShaders()

def enableMaskShaders():
	print "edis"
	bake.enableMaskShaders()
