import maya.cmds as cmds
import maya.mel as mel
import os, sys
import denreiMaya

#check for nonunique names
#unify mesh, anim export

def run( ):
	print 'Beginning ActorX Export'
	bCreateIni = denreiMaya.getInfo( "bCreateIni" )
	bBakeAnim = denreiMaya.getInfo( "bBakeAnim" )
	bReposition = denreiMaya.getInfo( "bReposition" )
	bExportMesh = denreiMaya.getInfo( "bExportMesh" )
	bExportAnim = denreiMaya.getInfo( "bExportAnim" )
	animList = denreiMaya.getAnimList()
	rootJoint = denreiMaya.getInfo('rootJoint')

	if bExportMesh == 1:
		actorXMesh()
	if bExportAnim == 1:
		actorXAnimations()
	return 'ActorXComplete'

def actorXMesh( ):
	print 'Exporting Meshes...'

	charName = denreiMaya.getInfo( 'charName' )
	charID = denreiMaya.getInfo( 'characterID' )
	famID = denreiMaya.getInfo( 'familyID' )
	description = denreiMaya.getInfo( 'description' )
	faction = denreiMaya.getInfo( 'faction' )
	portrait = denreiMaya.getInfo( 'portrait' )

	bCreateIni = denreiMaya.getInfo( "bCreateIni" )
	bBakeAnim = denreiMaya.getInfo( "bBakeAnim" )
	bReposition = denreiMaya.getInfo( "bReposition" )
	bToOrigin = denreiMaya.getInfo( "bToOrigin" )
	bExportMesh = denreiMaya.getInfo( "bExportMesh" )
	bExportAnim = denreiMaya.getInfo( "bExportAnim" )
	animList = denreiMaya.getAnimList()
	rootJoint = denreiMaya.getInfo('rootJoint')

	if checkActorXLoaded() is 0:
		return denreiMaya.problem( 'ActorXTool.mll is not loaded.' )

	if not cmds.objExists( rootJoint ):
		return denreiMaya.problem( 'Please select a root joint.' )

	#Check for duplicate bone names
	jointList = cmds.ls( typ='joint' )
	for joint in jointList:
		cnt = 0
		for other in jointList:
			if other == joint:
				cnt += 1
		if cnt > 1:
			return denreiMaya.problem('Duplicate joint names detected\nresolve and try again.') #add specifics, reposition joints

	#TODO redo this
	sceneName = mel.eval( 'basenameEx( "' + cmds.file( q=1, sn=1) + '") ' )
	packagesPath = cmds.workspace( q=1, sn=1) + '/packages'
	packageName = sceneName.rstrip( '0123456789' )	
	outputFolder = packagesPath + '/' + packageName

	UPKFileName = denreiMaya.getInfo('UPKFileName')
	upkName = UPKFileName

	if not os.access( outputFolder, os.F_OK ):
		os.makedirs( outputFolder )

	ini = '[UTGame.UTCustomChar_Data]\n'
	warnings = ''

	#Get rid of weightSourceLayer binds
	wsConDict = {}
	for node in cmds.editDisplayLayerMembers( 'weightSourceLayer', q=1 ) or []:
		shapes = cmds.listRelatives(node, shapes=1) or []
		for shape in shapes:
			shapePlug = shape +'.inMesh'
			if cmds.connectionInfo( shapePlug, ged=1 ) != '':
				sourcePlug = cmds.connectionInfo( shapePlug, sfd=1 )
				wsConDict[ node ] = ( sourcePlug, shapePlug )
				cmds.disconnectAttr( wsConDict[node][0], wsConDict[node][1])

	#Reposition to Origin if not a UT3 character
	if bToOrigin and not bReposition:
		print 'Setting'
		timeSlider = denreiMaya.TimeSlider()
		timeSlider.setCur( 0 )
		group = cmds.group( em=1, w=1, n='Tmp_PositionNode' )
		cmds.xform( group, ws=1, t=cmds.xform( rootJoint, q=1, t=1) )
		pos = cmds.xform( group, ws=1, q=1, t=1) 
		cmds.parent( denreiMaya.findRootJoint(), group )
		cmds.xform( group, ws=1, t=(0,0,0) )
		cmds.addAttr( group, ln='startLoc', dt='double3')
		cmds.setAttr( group+ ".startLoc", pos[0],pos[1],pos[2], typ='double3')

	if( bExportMesh == 1 ):
		cmds.axexecute(path=outputFolder, skinfile=upkName )

	#Restore position from origin if not a UT3 Character
	if bToOrigin and not bReposition:
		print 'Restoring'
		group = cmds.listRelatives( denreiMaya.findRootJoint(), p=1 )[0]
		startLoc = cmds.getAttr( group+'.startLoc' )[0]
		cmds.xform( group, ws=1, t=startLoc )
		cmds.parent( denreiMaya.findRootJoint(), w=1)
		cmds.delete( group )
		timeSlider.restore()

	#Put the mesh history back as it was
	for node in wsConDict:
		cmds.connectAttr( wsConDict[node][0], wsConDict[node][1])	

	print "Done exporting: " + outputFolder + " : " + upkName
	return



def actorXUTCharacter( ):
	print 'Exporting UT3 Character...'

	charName = denreiMaya.getInfo( 'charName' )
	charID = denreiMaya.getInfo( 'characterID' )
	famID = denreiMaya.getInfo( 'familyID' )
	description = denreiMaya.getInfo( 'description' )
	faction = denreiMaya.getInfo( 'faction' )
	portrait = denreiMaya.getInfo( 'portrait' )

	bCreateIni = denreiMaya.getInfo( "bCreateIni" )
	bBakeAnim = denreiMaya.getInfo( "bBakeAnim" )
	bReposition = denreiMaya.getInfo( "bReposition" )
	bExportMesh = denreiMaya.getInfo( "bExportMesh" )
	bExportAnim = denreiMaya.getInfo( "bExportAnim" )
	animList = denreiMaya.getAnimList()
	rootJoint = denreiMaya.getInfo('rootJoint')

	if checkActorXLoaded() is 0:
		return denreiMaya.problem( 'ActorXTool.mll is not loaded.' )
	#Check for duplicate bone names
	jointList = cmds.ls( typ='joint' )
	for joint in jointList:
		cnt = 0
		for other in jointList:
			if other == joint:
				cnt += 1
		if cnt > 1:
			return denreiMaya.problem('Duplicate joint names detected\nresolve and try again.') #add specifics, reposition joints

	#TODO redo this
	sceneName = mel.eval( 'basenameEx( "' + cmds.file( q=1, sn=1) + '") ' )
	packagesPath = cmds.workspace( q=1, sn=1) + '/packages'
	packageName = sceneName.rstrip( '0123456789' )	
	outputFolder = packagesPath + '/' + packageName

	if not os.access( outputFolder, os.F_OK ):
		os.makedirs( outputFolder )

	ini = '[UTGame.UTCustomChar_Data]\n'
	warnings = ''
	createdParts = []

	#Get rid of weightSourceLayer binds
	wsConDict = {}
	for node in cmds.editDisplayLayerMembers( 'weightSourceLayer', q=1 ) or []:
		shapes = cmds.listRelatives(node, shapes=1) or []
		for shape in shapes:
			shapePlug = shape +'.inMesh'
			if cmds.connectionInfo( shapePlug, ged=1 ) != '':
				sourcePlug = cmds.connectionInfo( shapePlug, sfd=1 )
				wsConDict[ node ] = ( sourcePlug, shapePlug )
				cmds.disconnectAttr( wsConDict[node][0], wsConDict[node][1])

	#Create a list of the connections which lead to the skinClusters.
	#This is used to remove skinning history from all meshes except the one being exported
	conDict = {}
	for node in  cmds.editDisplayLayerMembers( '_exportLayer', q=1 ) or []:
		shapes = cmds.listRelatives(node, shapes=1) or []
		for shape in shapes:
			shapePlug = shape +'.inMesh'
			if cmds.connectionInfo( shapePlug, ged=1 ) != '':
				sourcePlug = cmds.connectionInfo( shapePlug, sfd=1 )
				conDict[ node ] = ( sourcePlug, shapePlug )
				cmds.disconnectAttr( conDict[node][0], conDict[node][1])

	#Do the export
	for node in conDict:
		print node
		name = node.partition('__')[0]
		part = name.rstrip( '0123456789' )
		if part in createdParts:
			warnings += 'More than 1 PART_'+part+' is being created, so make sure you give '+node+' a unique PartID by hand... \n'
		else:
			createdParts.append(part)

		upkName = 'SK_CH_'+charName+'_'+name
		ini += '\n+Parts=(Part=PART_'+part+',ObjectName="'+packageName+'.'+upkName+'",PartID="'+charID+'",FamilyID="'+famID+'")'
		cmds.connectAttr( conDict[node][0], conDict[node][1] )
		print conDict[node][0], conDict[node][1]
		if( bExportMesh == 1 ):
			cmds.axexecute(path=outputFolder, skinfile=upkName )
		cmds.disconnectAttr( conDict[node][0], conDict[node][1])		

	#Put the mesh history back as it was
	for node in conDict:
		cmds.connectAttr( conDict[node][0], conDict[node][1])		
	for node in wsConDict:
		cmds.connectAttr( wsConDict[node][0], wsConDict[node][1])	

	if len(createdParts) == 0:
		return 	cmds.confirmDialog( 
				title="Export Results", 
				message= 'No meshes selected.'
				)

	#Create the Character Data String for the UTCustomChar.ini file
	defaultParts = [ 'Head','Facemask','Torso','ShoPad','Arms','Thighs','Boots' ]
	charData = 'FamilyID="'+famID+'"'
	for part in defaultParts:
		if part in createdParts:
			charData += ','+part+'ID="'+charID+'"'
		else:
			charData += ','+part+'ID="NONE"'
	if 'ShoPad' in createdParts:
		charData += ',bHasLeftShoPad=true,bHasRightShoPad=true'
	else:
		charData += ',bHasLeftShoPad=false,bHasRightShoPad=false'

	ini += '\n+Characters=(CharName="'+charName+'",Description="'+description+'",CharID="'+charID+'",bLocked="False",Faction="'+faction+'",PreviewImageMarkup="'+portrait+'",CharData=('+charData+'))'

	if ( denreiMaya.getInfo( "bCreateIni" ) == 1 ):
		UnpublishedCodaFolder = denreiMaya.getInfo( 'UTGamePath' ).rstrip('/') + '/Unpublished/CookedPC/Coda/'
		UTCustomCharFileFolder = UnpublishedCodaFolder + packageName +'/'
		UTCustomCharFileLoc = UTCustomCharFileFolder + 'UTCustomChar.ini'

		if not os.access( UnpublishedCodaFolder, os.F_OK ):
			os.mkdir( UnpublishedCodaFolder )
		if not os.access( UTCustomCharFileFolder, os.F_OK ):
			os.mkdir( UTCustomCharFileFolder )
		if os.access( UTCustomCharFileLoc, os.F_OK ):
			os.remove( UTCustomCharFileLoc )

		UTCustomCharFile = open( UTCustomCharFileLoc, "w")
		UTCustomCharFile.write( ini )
		UTCustomCharFile.close()
	else:
		cmds.confirmDialog( 
		title="Export Results", 
		message= warnings + '\n\nUTCustomChar.ini created as\n'+UTCustomCharFileLoc+'\nThis UTCustomChar.ini template is in the script editor...\n'+ini
		)

	return '\n\n'+ini

def actorXAnimations():
	print 'Exporting Animations...'
	if checkActorXLoaded() is 0:
		return
	animationFolder = denreiMaya.getInfo( 'animationFolder' )
	if animationFolder == 'animationFolder':
		animationFolder = denreiMaya.getInfo( 'outputFolder' )
	if not os.access( animationFolder, os.F_OK ):
		os.makedirs( animationFolder )
	meshFolder = denreiMaya.getInfo( 'meshFolder' )
	if not os.access( meshFolder, os.F_OK ):
		os.makedirs( meshFolder )

	bCreateIni = denreiMaya.getInfo( "bCreateIni" )
	bBakeAnim = denreiMaya.getInfo( "bBakeAnim" )
	bReposition = denreiMaya.getInfo( "bReposition" )
	bToOrigin = denreiMaya.getInfo( "bToOrigin" )
	bExportMesh = denreiMaya.getInfo( "bExportMesh" )
	bExportAnim = denreiMaya.getInfo( "bExportAnim" )
	animList = denreiMaya.getAnimList()
	rootJoint = denreiMaya.getInfo('rootJoint')

	if( rootJoint is '' ):
		return denreiMaya.problem( 'Please set a root joint' )
	min = 999999
	max = -999999
	
	if len(animList) < 1:
		return denreiMaya.problem( 'Please add an animation with start and stop times' )

	for anim in animList:
		animMin = int( denreiMaya.getInfo( 'ianim_'+anim+'_min' ) )
		if min > animMin:
			min = animMin
		animMax = int( denreiMaya.getInfo( 'ianim_'+anim+'_max' ) )
		if max < animMax:
			max = animMax

	if bBakeAnim == 1:
		cmds.bakeResults(
			rootJoint,
			hi='both',
			simulation=1, 
			t= (min,max),
			sampleBy=1,
			disableImplicitControl=1,
			preserveOutsideKeys=1,
			sparseAnimCurveBake=0, 
			controlPoints=0,
			shape=1,
			)
		jointList = denreiMaya.getRootJointList()
		if rootJoint in jointList:
			jointList.remove( rootJoint )
		if len(jointList) > 0:
			cmds.delete( jointList )

	if bReposition == 1: #Do the reposition
		timeSlider = denreiMaya.TimeSlider()
		timeSlider.setCur( 0 )
		group = cmds.group( em=1, w=1, n='Tmp_PositionNode' )
		cmds.xform( group, ws=1, t=cmds.xform( rootJoint, q=1, t=1) )
		pos = cmds.xform( group, ws=1, q=1, t=1) 
		cmds.parent( denreiMaya.findRootJoint(), group )
		cmds.setAttr( group+'.rotateX', 90 )
		cmds.xform( group, ws=1, t=(0,0,0) )
		cmds.addAttr( group, ln='startLoc', dt='double3')
		cmds.setAttr( group+ ".startLoc", pos[0],pos[1],pos[2], typ='double3')

	jointList = cmds.ls( typ='joint' )
	for joint in jointList:
		cnt = 0
		for other in jointList:
			if other == joint:
				cnt += 1
		if cnt > 1:
			return denreiMaya.problem('Duplicate joint names detected\nresolve and try again.') #add specifics, reposition joints

	if bExportMesh == 1:
		skinFileName = denreiMaya.getInfo('skinFileName')
		cmd = 'axexecute -p "'+meshFolder+'" -sf "'+skinFileName+'" -tri'
		mel.eval( cmd )

	if bExportAnim == 1:
		animList = denreiMaya.getAnimList()
		for anim in animList:
			min = str(denreiMaya.getInfo( 'ianim_'+anim+'_min' ))
			max = str(denreiMaya.getInfo( 'ianim_'+anim+'_max' ))
			cmd =  'axexecute -seq "'+anim+'" -range '+min+'-'+max
			print cmd
			mel.eval( cmd )
		
		psaFileName = denreiMaya.getInfo('psaFileName')
		cmd = 'axexecute -p "'+animationFolder+'" -saveanim -animfile "'+psaFileName+'"'
		print cmd
		mel.eval( cmd )
		
	#TODO RESTORING POSITION DISABLED UNTIL UNDO NOT REQUIRED
	#Restore root joint position
	if bReposition == -999: #Restore from reposition
		group = cmds.listRelatives( denreiMaya.findRootJoint(), p=1 )[0]
		cmds.setAttr( group+'.rotateX', 0 )
		startLoc = cmds.getAttr( group+'.startLoc' )[0]
		cmds.xform( group, ws=1, t=startLoc )
		cmds.parent( denreiMaya.findRootJoint(), w=1)
		cmds.delete( group )
		timeSlider.restore()

	return

def checkActorXLoaded():
	loaded = cmds.pluginInfo( 'ActorX', q=1, l=1 )
	if loaded == 0:
		result = denreiMaya.problem( 'ActorXTool plugin not loaded... try to load?\n(This will mess up unless youve actually downloaded the actorx plugin from epic)' )
		if result == 'Confirm':
			return cmds.loadPlugin( 'ActorXTool' )
		else:
			return 0

def prepareAnimationExport():
	cmds.bakeResults(
				rootJoint,
				simulation=1,
				t=range, 
				sampleBy=1,
				disableImplicitControl=1,
				preserveOutsideKeys=1,
				sparseAnimCurveBake=0,
				controlPoints=0,
				shape=1,
				hi='below',
				)
