import denreiMaya
import util as dUtil
import maya.cmds as cmds
import maya.mel as mel
from pymel import *

OFFSET_DIST = 30
BAKE_SEARCH_OFFSET = 5
RELEASE = 0
OVERSCAN = 10

def enableMaskShaders():
	shaderNames = ['Color0','Color1','Color2','Color3','Color4']
	for shaderName in shaderNames:
		shaderNode = PyNode( shaderName )
		connectionList = shaderNode .listConnections(c=1,d=0,p=1,sourceFirst=1)
		for i, c in enumerate(connectionList):
			print c
			shaderNode.setAttr( 'SrcCon_'+str(i), c[0].name(), force=1 )
			shaderNode.setAttr( 'DstCon_'+str(i), c[1].name(), force=1 )
			disconnectAttr( c[0].name(), c[1].name() )
			shaderNode.setAmbientColor( [1,1,1] )
			shaderNode.setAttr( 'diffuse', 1 )
	SCENE.Color0.setColor( [0,0,0] )
	SCENE.Color1.setColor( [1,0,0] )
	SCENE.Color2.setColor( [0,1,0] )
	SCENE.Color3.setColor( [0,0,1] )
	SCENE.Color4.setColor( [1,1,1] )

def disableMaskShaders():
	shaderNames = ['Color0','Color1','Color2','Color3','Color4']
	for shaderName in shaderNames:
		shaderNode = PyNode( shaderName )
		i=0
		while( i >= 0 ):
			shaderNode.setAmbientColor( [0,0,0] )
			shaderNode.setAttr( 'diffuse', 0.8 )
			if shaderNode.hasAttr( 'SrcCon_'+str(i) ) and shaderNode.hasAttr( 'DstCon_'+str(i) ):
				srcPlug = getAttr( shaderNode.name() + '.SrcCon_'+str(i) )
				dstPlug = getAttr( shaderNode.name() + '.DstCon_'+str(i) )

				print i, srcPlug, dstPlug
				connectAttr( srcPlug, dstPlug )
				i += 1
			else:
				i = -1
				print i, srcPlug, dstPlug

def configureShader( normalTextureNode, diffuseTextureNode, alphaTextureNode, shaderNode ):
	if cmds.objExists(shaderNode):
		shaderNode = shaderNode
	else:
		newShader = mel.eval( 'renderCreateBarCB -asShader "surfaceShader" lambert' )
		shaderNode = cmds.rename( newShader, shaderNode )

	if diffuseTextureNode != None: #Connect a diffuse map
		if cmds.connectionInfo( shaderNode + '.color', isDestination=True ):
			cmds.disconnectAttr( cmds.connectionInfo( shaderNode + '.color', sfd=1 ), shaderNode + '.color' )
		cmds.connectAttr( diffuseTextureNode + '.outColor', shaderNode + '.color', f=1 )
	else:
		if cmds.connectionInfo( shaderNode + '.color', isDestination=True ):
			cmds.disconnectAttr( cmds.connectionInfo( shaderNode + '.color', sfd=1 ), shaderNode + '.color' )
		cmds.setAttr( shaderNode+".color", 1, 1, 1, type='double3' )

	if alphaTextureNode != None: #Connect a alpha map
		if cmds.connectionInfo( shaderNode + '.transparency', isDestination=True ):
			cmds.disconnectAttr( cmds.connectionInfo( shaderNode + '.transparency', sfd=1 ), shaderNode + '.transparency' )
		cmds.connectAttr( alphaTextureNode + '.outTransparency', shaderNode + '.transparency', f=1 )
	else:
		if cmds.connectionInfo( shaderNode + '.transparency', isDestination=True ):
			cmds.disconnectAttr( cmds.connectionInfo( shaderNode + '.transparency', sfd=1 ), shaderNode + '.transparency' )
		cmds.setAttr( shaderNode+".transparency", 0, 0, 0, type='double3' )

	if normalTextureNode != None: #Connect a normal map
		if cmds.connectionInfo( shaderNode + '.normalCamera', isDestination=True ):
			bumpNode = cmds.connectionInfo( shaderNode + '.normalCamera', sfd=True ).partition('.')[0]
		else:
			bumpNode = cmds.shadingNode('bump2d',asUtility=1)
			cmds.connectAttr( bumpNode + '.outNormal', shaderNode + '.normalCamera', f=1 ) 
		cmds.setAttr( bumpNode + '.bumpInterp', 1 )
		if cmds.connectionInfo( bumpNode + '.bumpValue', isDestination=True ):
			cmds.disconnectAttr( cmds.connectionInfo( bumpNode + '.bumpValue', sfd=1 ), bumpNode + '.bumpValue' )
		cmds.connectAttr( normalTextureNode + '.outAlpha', bumpNode + '.bumpValue', f=1 )
	else:
		if cmds.connectionInfo( shaderNode + '.normalCamera', isDestination=True ):
			cmds.disconnectAttr( cmds.connectionInfo( shaderNode + '.normalCamera', sfd=1 ), shaderNode + '.normalCamera' )

	return shaderNode

"""
def bakeFinal( selection ):
	sceneName = mel.eval( 'basenameEx( "' + cmds.file( q=1, sn=1) + '") ' )
	texturesPath = cmds.workspace( q=1, sn=1) + '/textures'
	texName_d = texturesPath + '/' + sceneName.rstrip( '0123456789' ) + selection[0] + '_d.tga'

	textureNode = testBake( selection[0], 2048 ) # add high quality sampling
	# save testBake as _n

	# do diffuse
	# save diffuse
"""

def bakeExportHigh( selection ):
	textureNode = testBake( selection[0], 2048 )
	configureShader( textureNode, 'BakeShader' )

def bakeDiffuseMap( target, size, ratio ):
	mapOutput = 'diffuseRGB'  #   litAndShadedRGB
	return bakeMap( target, mapOutput, size, ratio )

def bakeNormalMap( target, size, ratio ):
	mapOutput = 'normal'
	return bakeMap( target, mapOutput, size, ratio )

def bakeAlphaMap( target, size, ratio ):
	mapOutput = 'alpha'
	return bakeMap( target, mapOutput, size, ratio )

def bakeMap( target, mapOutput, size, ratio ):
	if size == 0:
		return None
	extension = 'tga'
	projectDir = (cmds.file( q=1, sn=1)).partition('scenes/')[0]
	filenameNoExtension = projectDir + 'textures/_'+mapOutput+'Bake'
	filename = filenameNoExtension+'.' + extension

	msg = 'No bakeNode found... empty and reassign contents of "_highResProxyLayer" with only meshes and make sure a bakeNode exists from MigrateMeshes'
	if cmds.objExists( 'bakeNode' ) == 0:
		cmds.confirmDialog( title='Denrei Requests...', message=msg )
		return

	dUtil.cleanLayerConnections( '_highResProxyLayer' )
	sourceList = cmds.editDisplayLayerMembers( '_highResProxyLayer', q=1, fn=1 ) or []
	if len(sourceList) is 0:
		msg = """	Couldn't find any source objects in '_highResProxyLayer' with that selection.
					Check that the geometry is assigned directly to the intended layers (and not through a parent, for example)
		"""
		cmds.confirmDialog( title='Denrei Requests...', message=msg )
		return ''

	if cmds.objExists( target ):
		cmds.surfaceSampler( 
			target=target,
			uvSet='map1',
			searchOffset=denreiMaya.getInfo('fSearchDepth'),
			maxSearchDistance=0 ,
			searchCage="" ,
			source=sourceList,
			mapOutput=mapOutput ,
			mapWidth=size ,
			mapHeight=size*ratio ,
			max=1 ,
			mapSpace='tangent' ,
			mapMaterials=1 ,
			shadows=1 ,
			filename=filenameNoExtension ,
			fileFormat=extension ,
			superSampling=0 ,
			filterType=0 ,
			filterSize=1 ,
			overscan=OVERSCAN ,
			searchMethod=2 ,
			useGeometryNormals=0 ,
			ignoreMirroredFaces=0 ,
			flipU=0 ,
			flipV=0 
			)

		newTextureNode = None
		for fileNode in cmds.ls( typ='file'):
			if cmds.getAttr( fileNode+'.fileTextureName' ) == filename:
				newTextureNode = fileNode

		if newTextureNode == None:
			newTextureNode = cmds.shadingNode( 'file', asTexture=1, n='testBake')

		cmds.setAttr( newTextureNode + '.fileTextureName', filename, typ='string') 
		
		return newTextureNode

	return None

#Create TGAs for all PSD nodes present in the scene.
#These will be placed in /textures/SCENE_NAME without numerics
#Normal maps will have their green channel flipped
#Please make sure you are using true 'psdFileTex' nodes and not just 'file' nodes
def PSDsToTGAs(  ):
	outputFolder = getTextureOutputFolder()
	if not os.access( outputFolder, os.F_OK ):
		os.mkdir( outputFolder )

	psdNodes = cmds.ls( typ='psdFileTex' )
	for psdNode in psdNodes: 
		layerSet = cmds.getAttr( psdNode + '.layerSetName' )
		psdFullPath = cmds.getAttr( psdNode + '.fileTextureName ' )
		psdFileName = mel.eval( 'basenameEx( "' + psdFullPath + '") ' )

		if layerSet == 'normal':
			#Use a remapColor node to flip the green channel
			remapNode = mel.eval( "shadingNode -asUtility remapColor;" )
			cmds.setAttr( remapNode + ".green[0].green_Position", 0 )
			cmds.setAttr( remapNode + ".green[0].green_FloatValue", 1 )
			cmds.setAttr( remapNode + ".green[1].green_Position", 1 )
			cmds.setAttr( remapNode + ".green[1].green_FloatValue", 0 )
			
			cmds.connectAttr( psdNode + ".outColor", remapNode + ".color", force=1 )
			textureSize = cmds.getAttr( psdNode + '.osx' )

			convertedNode = cmds.convertSolidTx(
					remapNode,
					alpha = False,
					resolutionX = textureSize,
					resolutionY = textureSize,
					name = psdNode+layerSet,
					fileFormat = 'tga',
					fileImageName = '%s/%s_%s.tga' % (outputFolder, psdFileName, layerSet),
					force=True,
					samplePlane=True,
					samplePlaneRange=[ 0, 1, 0, 1 ],
					fillTextureSeams=True,
					backgroundMode='extend'
					)

			cmds.delete( remapNode )
			cmds.delete( convertedNode )
		
		else:
			cmds.psdExport( 
				ifn= psdFullPath, 
				ofn= '%s/%s_%s.tga' % (outputFolder, psdFileName, layerSet), 
				lsn= layerSet, 
				format='tga',
				alphaChannelIdx=0			
				)


	return 'all good'

	
