import maya.cmds as cmds
import maya.mel as mel
import denreiMaya

def buildDenreiWindow( tlc=(0,0) ):
	if cmds.window('denreiWindow', exists=1):
		cmds.deleteUI('denreiWindow', window=1)
	if ( cmds.windowPref( 'denreiWindow', exists=1 ) ):
		tlc = cmds.windowPref( 'denreiWindow', q=1, tlc=1 )	
		cmds.windowPref( 'denreiWindow', remove=1 )
	denreiWindow = window = cmds.window( 'denreiWindow', s=0, t="Denrei Window", iconName='Denrei', resizeToFitChildren=1, tlc=tlc )
	denreiMaya.setInfo( 'windowName', denreiWindow )

	column = cmds.rowColumnLayout(cw=(1,205)) #Main column

	animationColumn = buildAnimationColumn()
	descriptionColumn = buildDescriptionColumn(  )
	modelingColumn = buildModelingColumn()
	textureColumn = buildTextureColumn()
	optionsColumn = buildOptionsColumn()

	cmds.setParent( '..' ) #Main column
	cmds.separator()
	link = cmds.text( label='http://wiki.codamod.com/Denrei_Maya', al='center' )
	cmds.setParent( '..' ) #linkColumn

	cmds.showWindow( window )
	return window

def buildDescriptionColumn():
	column = cmds.columnLayout( ) #descriptionColumn
	denreiFrameLayout( 'Description' )
	column = cmds.columnLayout( ) #descriptionColumn
	textFieldList=[]
	for info in denreiMaya.defaultCharacterInfoList:
		textFieldList.append( denreiTextField( info ) )
	i = 0
	for i in range(len(denreiMaya.defaultCharacterInfoList)):
		if i + 1 < len(textFieldList):
			focusCmd = 'cmds.setFocus("' + textFieldList[i+1] + '")'
		else:
			focusCmd = 'cmds.setFocus("' + textFieldList[0] + '")'
		cmds.textField( textFieldList[i], e=1, ec=focusCmd )

	cmds.setParent( '..' ) #rowColumn
	cmds.setParent( '..' )
	cmds.setParent( '..' ) #descriptionColumn
	return column

def buildModelingColumn( ):
	column = cmds.columnLayout() 
	denreiFrameLayout( 'Modeling' )
	cmds.columnLayout() 
	
	cmds.rowColumnLayout( nc=2,cw=[(1,75),(2,125)] )
	cmds.text( l='UPK Filename' )
	tf = denreiTextField( 'UPKFileName' )
	cmds.setParent( '..' ) 

	cmds.rowColumnLayout( nc=2,cw=[(1, 100),(2,100)] )
	cmds.button( l='blowOut', c='denreiMaya.blowOutAll()', w=100 )
	cmds.button( l='blowIn', c='denreiMaya.blowInAll()', w=100 )
	cmds.setParent( '..' ) 

	if ( denreiMaya.getInfo( 'bReadyToModel' ) == 0 ):
		cmds.button( l='Init scene for modeling', c='denreiMaya.prepareModelingScene()', w=200 )
	else:	
		cmds.button( l='MigrateMeshes', c='denreiMaya.migrateMeshes()', w=200 )

	cmds.setParent( '..' ) 
	cmds.setParent( '..' ) 
	cmds.setParent( '..' )

	return column

def buildAnimationColumn( ):
	bReallyExport = denreiMaya.getInfo( "bReallyExport" )
	bBakeAnim = denreiMaya.getInfo( "bBakeAnim" )
	bReposition = denreiMaya.getInfo( "bReposition" )
	animList = denreiMaya.getAnimList()

	column = cmds.columnLayout() 
	denreiFrameLayout( 'Animation' )
	cmds.columnLayout()

	cmds.rowColumnLayout( nc=2,cw=[(1,75),(2,125)] )
	cmds.text( l='PSA Filename' )
	tf = denreiTextField( 'psaFileName' )
	cmds.textField( tf, e=1, text=denreiMaya.getInfo('psaFileName').partition('.')[0]+'.psa' )
	cmds.setParent( '..' ) 

	cmds.rowColumnLayout( 
				numberOfColumns=4, 
				columnWidth=[(1, 120), (2, 30),(3,30),(4,20)],
				)
	tf = cmds.text( l='     Sequence Name' )
	tf = cmds.text( l=' Min')
	tf = cmds.text( l=' Max' )
	tf = cmds.text( l='' )
	for name in animList:
		tf = cmds.text( l=name, al='center' )
		intf = denreiIntField( 'ianim_'+name+'_min' )
		intf = denreiIntField( 'ianim_'+name+'_max' )
		removeButton = cmds.button( l='X', w=20 )
		cmds.button( removeButton, e=1, c='denreiMaya.replaceAnim("'+name+'","")' )
	cmds.setParent('..')

	addSequenceButton = cmds.button( l='Add Sequence', w=200 )
	cmds.button( addSequenceButton, e=1, c='denreiMaya.addAnim()' )

	ann='Swap keyframe values between nodes prefixed with "L_" and "R_" at current time'
	addSequenceButton = cmds.button( l='MirrorKeyFrames', w=200, ann=ann )
	cmds.button( addSequenceButton, e=1, c='denreiMaya.mirrorKeyFrames(cmds.ls(sl=1))' )

	cmds.rowColumnLayout(	#Cycle Row
				numberOfColumns=4, 
				columnWidth=[(1, 100), (2, 30),(3,30),(4,40)],
				)
	tf = cmds.text( l='Cycle Animation', al='center' )
	intf = denreiIntField( 'iCycle_min' )
	intf = denreiIntField( 'iCycle_max' )
	cycleButton = cmds.button( l='Cycle', w=20 )
	ann='Create a cycle from the indicated keyframes... this DESTROYS keyframes to follow the cycle!!!'
	cmds.button( cycleButton, e=1, c='denreiMaya.cycleAnimation()', ann=ann )
	cmds.setParent('..') # Cycle Row

	cmds.setParent( '..' )
	cmds.setParent( '..' )
	cmds.setParent( '..' )

	return column

def	buildTextureColumn():
	column = cmds.columnLayout()
	denreiFrameLayout( 'Texture' )
	cmds.rowColumnLayout( nc=1,columnWidth=[(1, 200)] )

	denreiCheckBox( 'bKeepShaders', 'Preserve shading networks' )
	denreiCheckBox( 'bMaskShaders', 'Mask shaders' )
	cmds.rowColumnLayout( nc=2,cw=[(1, 100),(2,100)] )
	cmds.text( l='Search Depth', al='center' )
	denreiFloatField( 'fSearchDepth' )
	cmds.text( l='NormalMap Size', al='center' )
	denreiIntField( 'iNormalMapSize' )
	cmds.text( l='Diffuse Size', al='center' )
	denreiIntField( 'iDiffuseMapSize' )
	cmds.text( l='Alpha Size', al='center' )
	denreiIntField( 'iAlphaMapSize' )
	cmds.text( l='Texture Height Ratio', al='center' )
	denreiIntField( 'iTextureHeightRatio' )	
	cmds.setParent( '..' )

	cmds.button( l='Bake Maps', c='denreiMaya.bakeNodes()', w=200 )
	cmds.button( label='Export PSDs', c='denreiMaya.exportPSD()', w=200, en=0 )

	cmds.setParent( '..' ) 
	cmds.setParent( '..' ) 
	cmds.setParent( '..' )
	return column

def	buildOptionsColumn():
	column = cmds.columnLayout() 
	denreiFrameLayout( 'ActorXExport' )
	cmds.rowColumnLayout( nc=1,columnWidth=[(1, 200)] )
	if( denreiMaya.getInfo( 'bCustomFolders' ) == 0 ):
		cmds.rowColumnLayout( nc=2,cw=[(1, 20),(2,180)] )
		cmds.button( label='+', c='denreiMaya.setInfo("bCustomFolders",1)', w=200 )
		tf = denreiTextField( 'outputFolder' )
		cmds.textField( tf, e=1, w=180 )
		cmds.setParent( '..' ) 
		
	else:
		denreiCheckBox( 'bCustomFolders', 'Customize output folders?' ) 
		cmds.rowColumnLayout( nc=2,columnWidth=[(1, 30),(2,170)] )
		folders = [ 'animationFolder','meshFolder','textureFolder' ]
		for folder in folders:
			cmds.text( l=folder.partition('Folder')[0] )
			tf = denreiTextField( folder )
			cmds.textField( tf, e=1,text='Default '+folder, w=170 )
		cmds.setParent( '..' ) 	
	
#	denreiCheckBox( 'bCreateIni', 'Create UTCustomChar.ini?' ) 
#	if( denreiMaya.getInfo( 'bCreateIni' ) == 1 ):
#		cmds.rowColumnLayout( nc=2,cw=[(1, 50),(2,150)] )
#		cmds.text( l='UTGame' )
#		denreiTextField( 'UTGamePath' )
#		cmds.setParent( '..' )

	denreiCheckBox( 'bExportMesh', 'Export Mesh' )
	denreiCheckBox( 'bExportAnim', 'Export Animation' )
	denreiCheckBox( 'bBakeAnim', 'Bake Animation' )	
	denreiCheckBox( 'bReposition', 'Position as UT3 Character' )
	denreiCheckBox( 'bToOrigin', 'Position to origin' )

	cmds.rowColumnLayout( nc=2,cw=[(1, 50),(2,150)] )
	cmds.text( l='UTGame' )
	denreiTextField( 'UTGamePath' )
	cmds.setParent( '..' )

	cmd='denreiMaya.setRootJointSceneInfoFromSelection();'
	denreiTextFieldButtonGrp( 'rootJoint','Select', cmd, 0 )

	cmds.button( l='Run ActorX', c='denreiMaya.runActorX()', w=200 )

	cmds.setParent( '..' )
	cmds.setParent( '..' ) 
	cmds.setParent( '..' )
	return column

def denreiTextField( info ):
	tf = cmds.textField( info )
	cmd = 'denreiMaya.setInfo("'+info+'", cmds.textField("'+tf+'",q=1,text=1));'
	cmd += 'cmds.textField("'+tf+'",e=1, text=denreiMaya.getInfo("'+info+'"));'
	cmds.textField( tf, e=1, w=200, cc=cmd, text=denreiMaya.getInfo( info ) )
	return tf

def denreiTextFieldButtonGrp( info, buttonLabel, bCmd, bEdit=1, cw=(60,100,25) ):
	tf = cmds.textFieldButtonGrp( info, label=info, 
							text=denreiMaya.getInfo( info ), 
							cw3=cw,
							w=200,
							buttonLabel=buttonLabel, 
							buttonCommand=bCmd,
							editable=bEdit,
							) #cc=cmd
	cmd = 'denreiMaya.setInfo("'+info+'", cmds.textFieldButtonGrp("'+tf+'",q=1,text=1));'
	cmd += 'cmds.textFieldButtonGrp("'+tf+'",e=1, text=denreiMaya.getInfo("'+info+'"));'
	cmds.textFieldButtonGrp( tf, e=1, cc=cmd )
	return tf
							
def denreiIntField( info ):
	tf = cmds.intField( info )
	cmd = 'denreiMaya.setInfo("'+info+'", cmds.intField("'+tf+'",q=1,v=1));'
	cmd += 'cmds.intField("'+tf+'",e=1, v=int(denreiMaya.getInfo("'+info+'")));'
	cmds.intField( tf, e=1, cc=cmd, v=int(denreiMaya.getInfo( info ) ) )
	return tf

def denreiFloatField( info ):
	tf = cmds.floatField( info )
	cmd = 'denreiMaya.setInfo("'+info+'", cmds.floatField("'+tf+'",q=1,v=1));'
	cmd += 'cmds.floatField("'+tf+'",e=1, v=float(denreiMaya.getInfo("'+info+'")));'
	cmds.floatField( tf, e=1, cc=cmd, v=float(denreiMaya.getInfo( info ) ) )
	return tf

def denreiCheckBox( info, label ):
	bVal = int(denreiMaya.getInfo( info ))
	cb = cmds.checkBox( info )
	cmds.checkBox( cb, e=1, 
					label=label, align='left', w=200,
					onc='denreiMaya.setInfo( "'+info+'", 1 )',
					ofc='denreiMaya.setInfo( "'+info+'", 0 )',
					v=bVal
					)
def denreiFrameLayout( label ):
	fl = cmds.frameLayout( label,cll=1,cl=denreiMaya.getInfo( 'i_'+label+'Column' ) )
	cmd = 'denreiMaya.setInfo("i_'+label+'Column",cmds.frameLayout("'+fl+'",q=1,cl=1));'
	cmd += 'denreiMaya.buildDenreiWindow();'
	cmds.frameLayout( fl, e=1, cc=cmd, ec=cmd )
	

