import maya.cmds as cmds
import maya.mel as mel
import denreiMaya

def createHighResProxy( meshes ):
	layerContents = cmds.editDisplayLayerMembers( 'highResWorkLayer', q=1 ) or []
	cmds.editDisplayLayerMembers('defaultLayer', cmds.editDisplayLayerMembers('highResWorkLayer',fn=1, q=1 ),  nr=1)
	cmds.editDisplayLayerMembers('highResWorkLayer', layerContents,  nr=1)

	newMeshes = cmds.duplicate( meshes )
	for mesh in newMeshes:
		if( len(cmds.listRelatives( mesh, s=1 )) < 1 ):
			cmds.delete( mesh )
			newMeshes.remove( mesh )

	createdMeshes = []
	for mesh in newMeshes:
		mesh = createIndividualHighResProxy( mesh )
#		cmds.delete( mesh, ch=1 )[0][0]
#		createdMeshes.append( mesh )

	return createdMeshes

def createHighResProxyUnified( meshes ):
	newMeshes = cmds.duplicate( meshes )
	for mesh in newMeshes:
		if( len(cmds.listRelatives( mesh, s=1 )) < 1 ):
			cmds.delete( mesh )
			newMeshes.remove( mesh )

	if len( newMeshes ) > 1:
		newMeshes = cmds.polyUnite( newMeshes )
	newMesh = newMeshes[0]
	cmds.delete( newMesh, ch=1 )

	return createIndividualHighResProxy( newMesh )

def createIndividualHighResProxy( mesh ):
	print mesh + '->createHighResProxy'

	if len( cmds.ls( mesh, typ='shape' ) or [] ) > 0:  #Will be empty list or string
		mesh = cmds.listRelatives( mesh, p=1 )[0]

	cmds.editDisplayLayerMembers( '_highResProxyLayer', mesh, nr=1 )
	cmds.xform( mesh, r=1, t=denreiMaya.getDistanceVector( 'highResWorkLocator', 'bakeLocator' ) )

	cmds.select( mesh, r=1 )
	mesh = cmds.polySmooth(
			method=0,
			divisions=2,
			continuity=1,
			keepBorder=0,
			keepSelectionBorder=0,
			keepHardEdge=0,
			keepTessellation=1,
			keepMapBorders=0,
			smoothUVs =1,
			propagateEdgeHardness=1,
			subdivisionLevels=2,
			divisionsPerEdge=1,
			pushStrength  =0.6,
			roundness=1, 
			constructionHistory=1, 
			)

	if cmds.listRelatives( mesh, p=1 ) is not None:
		cmds.parent( mesh, world=True) 

	return [mesh]
	
def createBakeMesh( meshes ):
	bakeNodes = denreiMaya.getChildrenFromLayer( '_bakeLayer' )
	if len(bakeNodes) > 0:
		cmds.delete( bakeNodes )

#	diffuseBakeNode = createDiffuseBakeMesh( meshes )

	bakeNodes = cmds.duplicate( meshes, rr=1 )

	if len( bakeNodes ) > 1:
		bakeNodes = cmds.polyUnite( bakeNodes )
	bakeNode = bakeNodes[0]
	cmds.xform( bakeNode, r=1, t=denreiMaya.getDistanceVector( 'lowResWorkLocator', 'bakeLocator' ) )

	cmds.polyTriangulate( bakeNode )
#	cmds.polySoftEdge( bakeNode, a=180, ch=1 )
	cmds.editDisplayLayerMembers( '_bakeLayer', bakeNode, nr=1 )
	bakeNode = cmds.rename(bakeNode, 'bakeNode')
	cmds.delete( bakeNode, ch=1 )
	print bakeNode
	return [bakeNode]

def createDiffuseBakeMesh( meshes ):
	bakeNodes = cmds.duplicate( meshes, rr=1 )

	if len( bakeNodes ) > 1:
		for node in bakeNodes:
			cmds.setAttr( node + '.t', 0, 0, 0, typ='double3' ) #Put the mesh back from blown-out
		bakeNodes = cmds.polyUnite( bakeNodes )
	bakeNode = bakeNodes[0]
	cmds.xform( bakeNode, r=1, t=denreiMaya.getDistanceVector( 'lowResWorkLocator', 'bakeLocator' ) )

	cmds.polyTriangulate( bakeNode )
#	cmds.polySoftEdge( bakeNode, a=180, ch=1 )
	cmds.editDisplayLayerMembers( '_bakeLayer', bakeNode, nr=1 )
	bakeNode = cmds.rename(bakeNode, 'diffuseBakeNode')
	cmds.delete( bakeNode, ch=1 )
	return [bakeNode]	

def createExportMesh( meshes ):
	weightMesh = getWeightMesh()
	rootJoint = denreiMaya.findRootJoint()

	exportNodes = cmds.editDisplayLayerMembers( '_exportLayer', q=1 ) or []
	if len(exportNodes) > 0:
		cmds.delete( exportNodes )
	
	for mesh in meshes:
		exportMesh = cmds.duplicate( mesh, n=mesh+'__Export', rr=1 )[0]
		cmds.setAttr( exportMesh + '.t', 0, 0, 0, typ='double3' ) #Put the mesh back from blown-out
		cmds.xform( exportMesh, r=1, t=denreiMaya.getDistanceVector( 'lowResWorkLocator', 'jointLocator' ) )
		cmds.select( exportMesh, r=1 )
		cmds.polyTriangulate( exportMesh )
		exportMesh = checkDoMirror( exportMesh )
		if( len(cmds.ls( exportMesh, typ='shape' )) > 0 ):
			exportMesh = cmds.listRelatives( exportMesh, p=1)[0]
		cmds.editDisplayLayerMembers( '_exportLayer', exportMesh, nr=1 )

		cmds.delete( exportMesh, ch = 1 )

		cluster =  cmds.skinCluster( rootJoint, exportMesh, mi=3, omi=0 )[0]

		if weightMesh != '':
			weightMeshClusters = cmds.ls( ( cmds.listHistory( weightMesh ) ), typ='skinCluster' ) or []
			if len(weightMeshClusters) > 0:
				weightMeshCluster = weightMeshClusters[0]
				if weightMesh != '':
					cmds.copySkinWeights(
						ss=	weightMeshCluster,
						ds=cluster,
						noMirror=1, 
						surfaceAssociation = 'closestPoint',
						influenceAssociation = 'name'
						)
		exportNodes.append( exportMesh )

	return exportNodes

def checkDoMirror( src ):
	for i in range(cmds.polyEvaluate( src, v=1)):
		if cmds.xform( src+'.vtx['+str(i)+']', t=1, ws=1, q=1 )[0] < 0:
			return src

	mirror = cmds.polyMirrorFace( src, p=(0,0,0), ws=1, direction=1, mergeMode=1 )
	src = cmds.connectionInfo( mirror[0]+'.output', dfs=1)[0].partition('.')[0]
	cmds.select( cl=1 )
#	cmds.polySoftEdge(src, a=180 )
	return src

def getWeightMesh():
	denreiMaya.cleanLayerConnections( 'weightSourceLayer' )
	srcWeightList = denreiMaya.getChildrenFromLayer( 'weightSourceLayer' )
	if len(srcWeightList) == 0:
		return ''

	rootJoint = denreiMaya.findRootJoint()
	srcWeightList = moveToNewSkel( rootJoint, srcWeightList )
	print srcWeightList
	if len(srcWeightList) > 1:
		srcWeight = cmds.polyUnite( srcWeightList, ch=1 )[0]
	else:	
		srcWeight = srcWeightList[0]
	skinClusterCount = len( cmds.ls( cmds.listHistory( srcWeight ), typ='skinCluster' ) )
	if skinClusterCount == 0:
		return cmds.confirmDialog( title='Denrei Requests...', message='Please check the weightSourceLayer.\nI cant find any skinClusters...' )		
	if skinClusterCount > 1:
		srcWeight = mergeSkinClusters(srcWeight)
		srcWeight = moveToNewSkel( rootJoint, [ srcWeight ] )[0]
		srcWeightList = cmds.editDisplayLayerMembers( 'weightSourceLayer', q=1 ) or []
		if len(srcWeightList) > 0:
			cmds.delete( srcWeightList )
		
	cmds.editDisplayLayerMembers( 'weightSourceLayer', srcWeight, nr=1 )
	srcWeight = cmds.rename( srcWeight, 'WeightSourceObject' )
	return srcWeight


def mergeSkinClusters( src ):
	srcClusterList = cmds.ls( ( cmds.listHistory( src ) ), typ='skinCluster' )
	if len(srcClusterList) < 2:
		return src
	srcJoint = cmds.ls( ( cmds.listHistory( src ) ), typ='joint' )[0]
	dst = cmds.duplicate( src, rr=1 )[0]
	dstCluster = cmds.skinCluster( srcJoint, dst, tsb=1 )[0] #mi=3, omi=0
	joints = cmds.listRelatives( srcJoint, ad=1 )
	cmds.skinCluster( dstCluster ,e=1, lw=1, wt=0, ai=joints )
	cmds.setAttr( dstCluster + '.normalizeWeights', 0 )

	#Copy weights --------------
	cmds.waitCursor(state=1)	
	progressBar = 0
	cmds.progressWindow(t='Merging Skin Clusters',pr=progressBar,st='Setting up...',ii=False )	
#---
	ch = cmds.constructionHistory( q=1, tgl=1 )
	if ch > 0: cmds.constructionHistory( tgl=0 )
	cpm = cmds.createNode( 'closestPointOnMesh' )
	cmds.connectAttr( cmds.listRelatives( src, s=1 )[0] + '.outMesh', cpm + '.inMesh', f=1 )
	
	srcClusterDict={}
	for srcCluster in srcClusterList:
		srcShapesOrig = cmds.ls( cmds.listHistory( srcCluster ), typ='mesh' )
		if len(srcShapesOrig) != 1:
			cmds.progressWindow(endProgress=1)	
			cmds.confirmDialog(title="Problem...",message='This history is too complex!\nClean it up as best you can :-/')
			cmds.delete( cpm )
			cmds.delete( dst )
			cmds.constructionHistory( tgl=1 )
			cmds.waitCursor(state=0)	
			return src

		srcShapeOrig = srcShapesOrig[0]
		cmds.connectAttr( srcShapeOrig + '.outMesh', cpm + '.inMesh', f=1 )	

		cmds.progressWindow(e=1,st='Finding '+srcCluster+' weights', max=cmds.polyEvaluate( dst, v=1 ))
		progressBar=0
		matchingVertList = {}
		for i in range( cmds.polyEvaluate( dst, v=1 ) ):
			vertName = dst+'.vtx['+str(i)+']'
			pos = cmds.xform( vertName, q=1, ws=1, t=1 )
			cmds.setAttr( cpm + '.ip', pos[0],pos[1],pos[2], typ='double3')
			otherVertIndex = cmds.getAttr( cpm + '.vt' )
			otherVertName = srcShapeOrig+'.vtx['+str(otherVertIndex)+']'
			if posIsEqual( pos, cmds.xform( otherVertName, q=1, ws=1, t=1 )):
				matchingVertList[i] = otherVertIndex
			progressBar += 1
			cmds.progressWindow(e=1,pr=progressBar)	
			
		srcClusterDict[srcCluster] = matchingVertList

	#Initialize weights on new object to 0 ---------
	cmds.skinPercent( dstCluster, dst+'.vtx[0:'+str(cmds.polyEvaluate( dst, v=1 ))+']', tv=(srcJoint,0) )

	for srcCluster in srcClusterDict:
#		cmds.progressWindow(e=1,st='Applying '+srcCluster+' weights', max=len(srcClusterDict[srcCluster]))
		progressBar=0
		for vert in srcClusterDict[srcCluster]:
			joints = cmds.skinCluster( srcCluster, inf=1, q=1 )	
			weightsCmd = 'cmds.skinPercent( '+srcCluster+', '+src+'.vtx['+str(srcClusterDict[srcCluster][vert])+'], v=1, query=True )'
			weights = cmds.skinPercent( srcCluster, src+'.vtx['+str(srcClusterDict[srcCluster][vert])+']', v=1, query=True )
			tvString = ''
			for i in range(len(joints)):
				if weights[i] > 0:
					tvString += '("%s",%s)'%(joints[i], weights[i])
					if i < len(joints):
						tvString += ','
				
			cmd = 'cmds.skinPercent( "'+ dstCluster +'", "'+dst+'.vtx['+str(vert)+']", tv=('+ tvString +') )' # , r=1
			cmds.python( cmd )
			progressBar += 1
			cmds.progressWindow(e=1,pr=progressBar)	
			

	#Finish up ---------------------

#	cmds.skinPercent( dstCluster, normalize=1)
	cmds.setAttr( dstCluster + '.normalizeWeights', 1 )
	cmds.constructionHistory( tgl=1 )
	cmds.delete( cpm )
	cmds.delete( src )
	cmds.progressWindow(endProgress=1)
	cmds.waitCursor(state=0)
	return dst

def moveToNewSkel( newRoot='b_Root', selection=[] ):
	if len(selection) == 0:
		selection = cmds.ls(sl=1,l=1) 

	finished = []
	for selected in selection:
		if cmds.nodeType( selected ) != 'transform':
			break
		
		srcMesh = selected 
		dstMesh = cmds.duplicate( srcMesh, n='toNewSkelTmp' )[0]

		for attr in cmds.listAttr(dstMesh, l=1) or []:
			cmds.setAttr( dstMesh+'.'+attr, lock=False )

		if cmds.listRelatives( dstMesh, p=1 ) is not None:
			cmds.parent( dstMesh, world=1 )
		cmds.makeIdentity( dstMesh, apply=1, t=1, r=1, s=1, n=0 )

		dstMesh = mergeSkinClusters( dstMesh )

		oldClusterList = cmds.ls( ( cmds.listHistory( srcMesh ) ), typ='skinCluster' ) or []
		if len(oldClusterList) == 0:
			return cmds.confirmDialog( title='Denrei Requests...', message='I cant find any clusters on '+srcMesh+'\nin "moveToNewSkel"...' )
		else:
			oldCluster = oldClusterList[0]
		if oldCluster != None:
			newCluster = cmds.skinCluster(
				newRoot, dstMesh,
			#	prune=1,
				maximumInfluences=3,
				obeyMaxInfluences=True,
				dropoffRate=4.0
				)[0]

			cmds.copySkinWeights( 
				ss=oldCluster,
				ds=newCluster,
				noMirror=1, 
				surfaceAssociation = 'closestPoint',
				influenceAssociation = 'name'
				) 

		cmds.delete( srcMesh )

		dstMesh = cmds.rename( dstMesh, srcMesh )
		finished.append( dstMesh )
	return finished

def posIsEqual( pos1, pos2, delta=0.005 ):
	if abs(pos1[0]) < delta and abs(pos1[1]) < delta and abs(pos1[2]) < delta:
		return 0
	if abs(pos2[0]) < delta and abs(pos2[1]) < delta and abs(pos2[2]) < delta:
		return 0
	for i in range(3):
		if abs(pos1[i] - pos2[i]) > delta:
			return 0
	return 1

