import maya.cmds as cmds
import maya.mel as mel
import sys
import os
import denreiMaya

def problem( msg='Cannot complete',title='Problem!'):
	cmds.confirmDialog( title=title, message=msg )
	return 'problem'

def setInfo( info, val ):
	if ( cmds.attributeQuery( info, node=denreiMaya.sceneInfoNode, ex=1 ) == 0 ):
		cmds.addAttr( denreiMaya.sceneInfoNode, ln=info, dt="string")
	
	cmds.setAttr( denreiMaya.sceneInfoNode+'.'+info, val, typ="string")

	if info is 'bMaskShaders':
		if val is 0:
			denreiMaya.disableMaskShaders()
		else:
			if val is 1:
				denreiMaya.enableMaskShaders()

	if info in denreiMaya.rebuildNotifyList:
		denreiMaya.buildDenreiWindow()

def getInfo( info ):
	if ( cmds.attributeQuery( info, node=denreiMaya.sceneInfoNode, ex=1 ) == 0 ):
		cmds.addAttr( denreiMaya.sceneInfoNode, ln=info, dt="string")
		cmds.setAttr( denreiMaya.sceneInfoNode+'.'+info, '', typ="string")

	val = cmds.getAttr( denreiMaya.sceneInfoNode+'.'+info ) or ''

	if ( info[0] == 'f' ):
		if val is '0' or val is '':
			return 0
		else:
			return float( val )

	if ( info[0] == 'b' or info[0] == 'i'  ):
		if info is 'bReadyToModel':
			return denreiMaya.isReadyToModel()
		if val is '0' or val is '':
			return 0
		else:
			return int(val)

	if val is '':
		return getDefaultVal( info )

	if info == 'UTGamePath':
		return val.replace('\\','/')

	return val or ''

def getDefaultVal( info ):
	sceneName = mel.eval( 'basenameEx( "' + cmds.file( q=1, sn=1) + '") ' )
	packagesRoot = cmds.workspace( q=1, sn=1) + '/packages'
	packageName = sceneName.rstrip( '0123456789' )

	if( info.find('Folder') > -1 ):
		if ( info == 'outputFolder' ):
			return packagesRoot + '/' + packageName.lstrip('CH_').lstrip('VH_').lstrip('AS_')
		elif ( info == 'textureFolder' or info == 'animationFolder' or info == 'meshFolder' ):
			return info
	elif( info.find('File') > -1 ):
		if ( info == 'skinFileName' ):
			return 'SK_'+sceneName.rstrip( '0123456789' ).lstrip('CH_').lstrip('VH_').lstrip('AS_')
		elif ( info == 'psaFileName' ):
			return 'AS_'+sceneName.rstrip( '0123456789' ).lstrip('CH_').lstrip('VH_').lstrip('AS_')
	elif ( info.find('portrait') == 0 ):
		return '<Images:UI_Portrait.Character.UI_Portrait_Character_Liandri_Male_Head02>'
	elif ( info.find('UTGamePath') == 0 ):
		return 'Enter UTGame path'
	elif ( info.find('charName') == 0 ):
		return 'Character Name'
	elif ( info.find('description') == 0 ):
		return 'Describe this character'
	elif ( info.find('characterID') == 0 ):
		return 'CharacterID'
	elif ( info.find('faction') == 0 ):
		return 'FactionName'
	elif ( info.find('familyID') == 0 ):
		return 'FamilyID'
	elif ( info.find('rootJoint') == 0 ):
		if len( cmds.ls( 'b_Root', typ='joint' ) ) > 0:
			return 'b_Root'			

	return ''

def addAnim( stringList='' ):
	animString = getInfo( 'animString' )
	if stringList == '':
		result = cmds.promptDialog(
				title='Add Sequence',
				message='Use spaces to separate the values\n[Name] [StartFrame] [EndFrame]',
				button=['OK', 'Cancel'],
				defaultButton='OK',
				cancelButton='Cancel',
				dismissString='Cancel')
		if result == 'OK':
			stringList = cmds.promptDialog(query=True, text=True).split()
	else:
		stringList = stringList.split()
		
	if len(stringList) is not 3:
		return problem('Improper values for animation sequence')
	text = ' '+stringList[0]+' '
	found = animString.find( text )
	if found <= 0:
		if not int(stringList[1]) < int(stringList[2]):
			return problem( 'Start must always be smaller than Stop' )
		setInfo( 'ianim_'+stringList[0]+'_min', stringList[1] )
		setInfo( 'ianim_'+stringList[0]+'_max', stringList[2] )
		setInfo( 'animString', (animString +text) )

def replaceAnim( oldName, newName ):
	result = ''
	if newName == '':
		result = cmds.promptDialog(
				title='Replace Sequence',
				message='Empty string for removal',
				text=oldName,
				button=['OK', 'Cancel'],
				defaultButton='OK',
				cancelButton='Cancel',
				dismissString='Cancel')
	if result == 'OK':
		newName = cmds.promptDialog(query=True, text=True)
	else:
		return

	min = str(denreiMaya.getInfo( 'ianim_'+oldName+'_min' ))
	max = str(denreiMaya.getInfo( 'ianim_'+oldName+'_max' ))
	addedAnim = addAnim( newName+' '+min+' '+max )
	if addedAnim == 'problem':
		return problem( 'Could not add new anim: '+newName )

	animString = getInfo( 'animString' )
	setInfo( 'animString', animString.replace( ' '+str(oldName)+' ','' ))

def getAnimList():
	return getInfo( 'animString' ).split()

def findRootJoint():
	rootJoints = []
	joints = cmds.ls( typ='joint' )
	for joint in joints:
		parents = cmds.listRelatives( joint, p=1 ) or []
		if len( cmds.ls( parents, typ='joint' ) or []) < 1:
			rootJoints.append( joint )
	if len(rootJoints) > 1:
		problem( 'Too many root joints, please make a single skeleton' )
		return ''
	elif len(rootJoints) < 1:
		problem( 'No joints found in this scene' )
		return ''
	return rootJoints[0]

def getRootJointList():
	rootJoints = []
	joints = cmds.ls( typ='joint' )
	for joint in joints:
		parents = cmds.listRelatives( joint, p=1 ) or []
		if len( cmds.ls( parents, typ='joint' ) or []) < 1:
			rootJoints.append( joint )
	return rootJoints

def setRootJointSceneInfoFromSelection():
	print 'selectingRootJoint'
	if len(getRootJointList()) > 1:
		if getInfo( 'bBakeAnim' ) is 0:
			problem('Too many root joints... you will have to bake') #broken
			setInfo( 'bBakeAnim', 1 )
	selection = cmds.ls(sl=1, typ='joint')
	if( len(selection) is not 1 ):
		problem( 'No joints/too many joints selected.' )
		setInfo( 'rootJoint', '' )
	else:
		setInfo( 'rootJoint', selection[0] )

def cleanLayerConnections( layer ):
	members = cmds.editDisplayLayerMembers( layer, q=1 ) or []
	
	#Move all layer connections off of shapes, let the shapes decide what layer they belong to
	shapes = cmds.ls( members, typ='shape' )
	for shape in shapes:
		parent = cmds.listRelatives( shape, p=1 )[0]
		oldLayerPlug = cmds.connectionInfo( parent+'.drawOverride', sfd=1 ) or []
		if len(oldLayerPlug) > 0: #Will be empty list or string
			cmds.disconnectAttr( oldLayerPlug, parent+'.drawOverride' )
		cmds.connectAttr( layer+'.drawInfo', parent+'.drawOverride' ) 
		cmds.disconnectAttr( layer+'.drawInfo', shape+'.drawOverride' )
			
def getLayerFromObject( node ):
	connectedLayers = []
	parents = (cmds.listRelatives( node, ap=1 ) or []) + [node]
	for	parent in parents:
		drawInfoPlug = cmds.connectionInfo( parent + '.drawOverride', sfd=1 ) or []
		if len( drawInfoPlug ) > 0: #Will be empty list or string
			connectedLayers.append( drawInfoPlug.partition('.')[0] )

	if len( connectedLayers ) < 1:
		return []
	if len( connectedLayers ) > 1:
		msg = self.name + ' is in too many layers:\n'
		for layer in connectedLayers:
			msg += '\n' + layer
		util.problem( msg )
	return connectedLayers[0]
	
def createLayer( name, colorNum=0, displayNum=0 ):
	if cmds.objExists( name ):
		layer = name
	else:
		layer = cmds.createDisplayLayer( name=name, number=1, empty=1 )
		cmds.setAttr( layer + '.displayType', displayNum )
	cmds.setAttr( layer + '.color', colorNum )
	return layer

def getChildrenFromLayer( layer, type='' ):
	members = cmds.editDisplayLayerMembers( layer, q=1 ) or []
	if len(members) > 0:
		members += cmds.listRelatives( members, ad=1 )
		if type != '':
			return cmds.ls( members, typ=type )
	return members

def isChildFromLayer( layer, node ):
	members = getChildrenFromLayer( layer )
	if node in members:
		return 1
	return 0

def getDistanceVector( originNode, destNode ):
	o = cmds.xform( originNode, q=1, rp=1, ws=1 )
	d = cmds.xform( destNode, q=1, rp=1, ws=1 )
	d[0] -= o[0]
	d[1] -= o[1]
	d[2] -= o[2]
	return d

def findUVPivot():
	
	uvs = cmds.polyEditUV( q=1 )

	x=y=i=0
	while i < len(uvs):
		x+= uvs[i]
		y+= uvs[i+1]
		i+=2
	uvCount = len(uvs) /2
	x = x/uvCount 
	y = y/uvCount 

	return (x, y)
	
def SelectionToComponentType( type ):
	mel.eval( 'setSelectMode components Components' )

	if type == 'f':
		cmds.select( cmds.polyListComponentConversion( tf=1, internal=1 ) or None, r=1 ) 
		mel.eval( 'selectType -smp 0 -sme 0 -smf 1 -smu 0 -pv 0 -pe 0 -pf 1 -puv 0;' )

	elif type == 'e':
		cmds.select( cmds.polyListComponentConversion( te=1, internal=1 ) or None, r=1 ) 
		mel.eval( 'selectType -smp 0 -sme 1 -smf 0 -smu 0 -pv 0 -pe 1 -pf 0 -puv 0;' )

	elif type == 'uv':
		cmds.select( cmds.polyListComponentConversion( tuv=1, internal=0 ) or None, r=1 ) 
		mel.eval( 'selectType -pv 0 -pe 0 -pf 0 -puv 1 -smp 0 -sme 0 -smf 0 -smu 1;' )

	elif type == 'v':
		cmds.select( cmds.polyListComponentConversion( tv=1, internal=0 ) or None, r=1 ) 
		mel.eval( 'selectType -smp 1 -sme 0 -smf 0 -smu 0 -pv 1 -pe 0 -pf 0 -puv 0;' )

		
def posIsEqual( pos1, pos2, delta=0.005 ):
	if abs(pos1[0]) < delta and abs(pos1[1]) < delta and abs(pos1[2]) < delta:
		return 0
	if abs(pos2[0]) < delta and abs(pos2[1]) < delta and abs(pos2[2]) < delta:
		return 0
	for i in range(3):
		if abs(pos1[i] - pos2[i]) > delta:
			return 0
	return 1



def mirrorUnrealJoint( selection ):
	attributes = [ '.translateZ', '.rotateX', '.rotateY' ]

	joints = []
	joints.extend( cmds.duplicate( selection[0], rr=1 ) )
	joints.extend( cmds.listRelatives( joints[0], allDescendents=1,typ='joint', f=1 ) )

	for joint in joints:
		for attribute in attributes:
			cmds.setAttr( joint + attribute, -1 * cmds.getAttr( joint + attribute ) )

	cmds.select( joints[0], r=1 )
	if joints[0].partition('Left')[1] == 'Left':
		mel.eval( 'searchReplaceNames "Left" "Right" "hierarchy";' )

	elif joints[0].partition('Right')[1] == 'Right':
		mel.eval( 'searchReplaceNames "Right" "Left" "hierarchy";' )

	elif joints[0].partition('L_')[1] == 'L_':
		mel.eval( 'searchReplaceNames "L_" "R_" "hierarchy";' )

	elif joints[0].partition('R_')[1] == 'R_':
		mel.eval( 'searchReplaceNames "R_" "L_" "hierarchy";' )

	selected = cmds.ls( sl=1)[0]
	cmds.rename( selected, selected.rstrip( '1234567890' ) )

	return """Looks like the bones were mirrored properly."""

def rebuildSkeletonEntry( self ):
	new = cmds.duplicate( self, n=self+'_new', rr=1 )[0]
	children = cmds.listRelatives( new, c=1, pa=1 ) or []
	for child in children:
		cmds.delete( child )
	rebuildSkeleton( self )

def rebuildSkeleton( self ):
	children = cmds.listRelatives( self, c=1) or []
	for child in children:
		cmds.select( self+'_new', r=1 )
		cmds.joint( p=cmds.xform(child,q=1,t=1,ws=1), o=cmds.joint(child,q=1,o=1), n=child+'_new' )
		rebuildSkeleton( child )

	old = self
	new = self+'_new'
	ra = cmds.xform( new, q=1, ra=1 )
	cmds.xform( new, ra=ra )
	cmds.delete( old )
	cmds.rename( new, old )

def createJointFromTransform( transformName ):

	pos = cmds.xform( transformName, q=1, t=1, ws=1 ) 
	newJoint = cmds.joint( p=pos, n=transformName+'YY')
	cmds.select( cl=1 )
	rot = cmds.xform( transformName, q=1, ro=1 ) 
	cmds.rotate( rot[0],rot[1],rot[2], newJoint )

	for child in cmds.listRelatives( transformName, c=1 ) or []:
		childJoint = createJointFromTransform( child )
		cmds.parent( childJoint, newJoint )

	return newJoint
	
def correctChild( self, excludeList ):
	children = cmds.listRelatives( self, c=1) or []
	childDict={}
	for child in children:
		childDict[child]=(cmds.xform(child,q=1,t=1,ws=1),cmds.xform(child,q=1,ro=1,ws=1))
	#Do reposition
	#	cmds.rotate(-90,0,0,self,r=1,os=1)
	cmds.xform( self, ro=(-90,-90,0), ws=1)
	cmds.setAttr( self+ '.jointOrient',0,0,0, typ='double3')

	for child in childDict:
		cmds.xform( child, ws=1, t=(childDict[child][0]))
		cmds.xform( child, ws=1, ro=(childDict[child][1]))
		if not child in excludeList:
			correctChild(child, excludeList)

def reconnectChildren( self ):
	children = cmds.listRelatives( self, c=1) or []
	for child in children:
		newJoint = cmds.disconnectJoint( child )
		cmds.delete( newJoint )
	for child in children:
		cmds.connectJoint( child, self, pm=True )

def resizeChild( self, s ):
	lockedList = cmds.listAttr( self, l=1 ) or []
	for attr in lockedList:
		cmds.setAttr( self+'.'+attr, l=0 )
	t = cmds.getAttr( self+'.t' )[0]
	cmds.setAttr( self+'.t', t[0]*s, t[1]*s, t[2]*s, typ='double3' )
	for attr in lockedList:
		cmds.setAttr( self+'.'+attr, l=1 )

	children = cmds.listRelatives( self, c=1 ) or []
	for child in children:
		resizeChild( child, s )

def toggleTime( jumpSize=500 ):
	time = cmds.currentTime( q=1 )
	if time > jumpSize:
		cmds.currentTime( time-jumpSize )
	else:
		cmds.currentTime( time+jumpSize )

class TimeSlider:
	def __init__( self, cur=None, min=None, max=None ):
		self._cur = cur or self.getCur()
		self._min = min or self.getMin()
		self._max = max or self.getMax()
	def getCur( self ):
		return cmds.currentTime(q=1)
	def getMin( self ):
		return cmds.playbackOptions(q=1,min=1)
	def getMax( self ):
		return cmds.playbackOptions(q=1,max=1)
	def setCur( self, time ):
		if self.getCur == time:
			return
		if self.getMin > time:
			cmds.playbackOptions( min=time-1 )
		elif self.getMax < time:
			cmds.playbackOptions( max=time+1 )
		cmds.currentTime( time )
	def setRange( self, min, max ):
		cmds.playbackOptions(e=1,min=self._min,max=self._max)		
	def restore( self ):
		cmds.playbackOptions(e=1,min=self._min,max=self._max)
		cmds.currentTime( self._cur )
